searchData={"items":[{"type":"module","title":"Cmp","doc":"Semantic comparison and sorting for Elixir.","ref":"Cmp.html"},{"type":"module","title":"Why `Cmp`? - Cmp","doc":"The built-in comparison operators as well as functions like `Enum.sort/2` or `Enum.max/1`\nare based on Erlang's term ordering and suffer two issues, which require attention and\nmight lead to unexpected behaviors or bugs:","ref":"Cmp.html#module-why-cmp"},{"type":"module","title":"1. Structural comparisons - Cmp","doc":"Built-ins use [structural comparison over semantic comparison](\n  https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n    ~D[2020-03-02] > ~D[2019-06-06]\n    false\n\n    iex> Enum.sort([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]])\n    [~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]]\n\nSemantic comparison is available but not straightforward:\n\n    iex> Date.compare(~D[2019-01-01], ~D[2020-03-02])\n    :lt\n\n    iex> Enum.sort([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]], Date)\n    [~D[2019-01-01], ~D[2019-06-06], ~D[2020-03-02]]\n\n`Cmp` does the right thing out of the box:\n\n    iex> Cmp.gt?(~D[2020-03-02], ~D[2019-06-06])\n    true\n\n    iex> Cmp.sort([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]])\n    [~D[2019-01-01], ~D[2019-06-06], ~D[2020-03-02]]","ref":"Cmp.html#module-1-structural-comparisons"},{"type":"module","title":"2. Weakly typed - Cmp","doc":"Built-in comparators accept any set of operands:\n\n    iex> 2 < \"1\"\n    true\n\n    iex> 0   false   Cmp.lte?(1, 1.0)\n    true\n\n    iex> Cmp.lte?(2, \"1\")\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 2, right: \"1\"","ref":"Cmp.html#module-2-weakly-typed"},{"type":"module","title":"What's in the box - Cmp","doc":"- Boolean comparisons: `eq?/2`, `lt?/2`, `gt?/2`, `lte?/2`, `gte?/2`\n- Equivalents of `Kernel.min/2`/ `Kernel.max/2`: `Cmp.min/2`, `Cmp.max/2`\n- Equivalents of `Enum.min/1`/ `Enum.max/1`/`Enum.sort/2`: `Cmp.min/1`, `Cmp.max/1`, `Cmp.sort/2`\n- `compare/2`","ref":"Cmp.html#module-what-s-in-the-box"},{"type":"module","title":"Supported types - Cmp","doc":"The `Cmp.Comparable` protocol is implemented for the following types:\n\n- `Integer`\n- `Float`\n- `Bitstring`\n- `Date`\n- `Time`\n- `DateTime`\n- `NaiveDateTime`\n- `Version`\n- `Tuple` (see below)\n- `Decimal` (if available)\n\nIt isn't implemented for atoms by design, since atoms are not semantically\nan ordered type.\n\nIt supports tuples of the same size and types:\n\n    iex> Cmp.max({12, ~D[2019-06-06]}, {12, ~D[2020-03-02]})\n    {12, ~D[2020-03-02]}\n\n    iex> Cmp.max({12, \"Foo\"}, {15, nil})\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: \"Foo\", right: nil\n\n    iex> Cmp.max({12, \"Foo\"}, {15})\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: {12, \"Foo\"}, right: {15}\n\n`Decimal` support can prevent nasty bugs too:\n\n    iex> max(Decimal.new(2), Decimal.from_float(1.0))\n    #Decimal<1.0>\n    iex> Cmp.max(Decimal.new(2), Decimal.from_float(1.0))\n    #Decimal<2>\n\nSee the `Cmp.Comparable` documentation to implement the protocol for other existing\nor new structs.","ref":"Cmp.html#module-supported-types"},{"type":"module","title":"Design goals - Cmp","doc":"- Fast and well-optimized - the overhead should be quite small over built-in equivalents.\n  See the `benchmarks/` folder for more details.\n- No need to require macros, plain functions\n- Easily extensible through the `Cmp.Comparable` protocol\n- Robust and well-tested (both unit and property-based)\n\nSupporting comparisons between non-homogeneous types such as mixed `Decimal` and\nbuilt-in numbers for instance is a non-goal. This limitation is a necessary\ntrade-off in order to ensure the points above. Use the `Decimal` library\ndirectly if you need this.","ref":"Cmp.html#module-design-goals"},{"type":"module","title":"Limitations - Cmp","doc":"- `Cmp` comparators cannot be used in guards.\n- `Cmp` does not support (or plan to support) comparisons between non-homogeneous types\n  (e.g. `Decimal` and native numbers).","ref":"Cmp.html#module-limitations"},{"type":"function","title":"Cmp.compare/2","doc":"Returns `:gt` if `left` is semantically greater than `right`, `lt` if `left`\nis less than `right`, and `:eq` if they are equal.","ref":"Cmp.html#compare/2"},{"type":"function","title":"Examples - Cmp.compare/2","doc":"iex> Cmp.compare(2, 1)\n    :gt\n\n    iex> Cmp.compare(1, 1.0)\n    :eq\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.compare(1, \"\")\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: \"\"","ref":"Cmp.html#compare/2-examples"},{"type":"function","title":"Cmp.eq?/2","doc":"Safe equivalent to `==/2`, which only works if both types are compatible and\nuses semantic comparison.","ref":"Cmp.html#eq?/2"},{"type":"function","title":"Examples - Cmp.eq?/2","doc":"iex> Cmp.eq?(2, 1)\n    false\n\n    iex> Cmp.eq?(1, 1.0)\n    true\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.eq?(1, \"\")\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: \"\"","ref":"Cmp.html#eq?/2-examples"},{"type":"function","title":"Cmp.gt?/2","doc":"Safe equivalent to `>/2`, which only works if both types are compatible and\nuses semantic comparison.","ref":"Cmp.html#gt?/2"},{"type":"function","title":"Examples - Cmp.gt?/2","doc":"iex> Cmp.gt?(2, 1)\n    true\n\n    iex> Cmp.gt?(1, 2)\n    false\n\n    iex> Cmp.gt?(1, 1.0)\n    false\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.gt?(1, nil)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nUnlike `>/2`, it will perform a semantic comparison for structs and not a\n[structural comparison](https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n    iex> Cmp.gt?(~D[2020-03-02], ~D[2019-06-06])\n    true\n\n    ~D[2020-03-02] > ~D[2019-06-06]\n    false","ref":"Cmp.html#gt?/2-examples"},{"type":"function","title":"Cmp.gte?/2","doc":"Safe equivalent to `>=/2`, which only works if both types are compatible and\nuses semantic comparison.","ref":"Cmp.html#gte?/2"},{"type":"function","title":"Examples - Cmp.gte?/2","doc":"iex> Cmp.gte?(2, 1)\n    true\n\n    iex> Cmp.gte?(1, 2)\n    false\n\n    iex> Cmp.gte?(1, 1.0)\n    true\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.gte?(1, nil)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nUnlike `>=/2`, it will perform a semantic comparison for structs and not a\n[structural comparison](https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n    iex> Cmp.gte?(~D[2020-03-02], ~D[2019-06-06])\n    true\n\n    ~D[2020-03-02] >= ~D[2019-06-06]\n    false","ref":"Cmp.html#gte?/2-examples"},{"type":"function","title":"Cmp.lt?/2","doc":"Safe equivalent to `</2`, which only works if both types are compatible and\nuses semantic comparison.","ref":"Cmp.html#lt?/2"},{"type":"function","title":"Examples - Cmp.lt?/2","doc":"iex> Cmp.lt?(1, 2)\n    true\n\n    iex> Cmp.lt?(2, 1)\n    false\n\n    iex> Cmp.lt?(1, 1.0)\n    false\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.lt?(1, nil)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nUnlike `</2`, it will perform a semantic comparison for structs and not a\n[structural comparison](https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n    iex> Cmp.lt?(~D[2019-06-06], ~D[2020-03-02])\n    true\n\n    ~D[2019-06-06] < ~D[2020-03-02]\n    false","ref":"Cmp.html#lt?/2-examples"},{"type":"function","title":"Cmp.lte?/2","doc":"Safe equivalent to `<=/2`, which only works if both types are compatible and\nuses semantic comparison.","ref":"Cmp.html#lte?/2"},{"type":"function","title":"Examples - Cmp.lte?/2","doc":"iex> Cmp.lte?(1, 2)\n    true\n\n    iex> Cmp.lte?(2, 1)\n    false\n\n    iex> Cmp.lte?(1, 1.0)\n    true\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.lte?(1, nil)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nUnlike `<=/2`, it will perform a semantic comparison for structs and not a\n[structural comparison](https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n    iex> Cmp.lte?(~D[2019-06-06], ~D[2020-03-02])\n    true\n\n    ~D[2019-06-06] <= ~D[2020-03-02]\n    false","ref":"Cmp.html#lte?/2-examples"},{"type":"function","title":"Cmp.max/1","doc":"Safe equivalent of `Enum.max/2`, returning the minimum of a non-empty\nenumerable of comparables.","ref":"Cmp.html#max/1"},{"type":"function","title":"Examples - Cmp.max/1","doc":"iex> Cmp.max([1, 3, 2])\n    3\n\nRespects semantic comparison:\n\n    iex> Cmp.max([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]])\n    ~D[2020-03-02]\n\nRaises a `Cmp.TypeError` on non-uniform enumerables:\n\n    iex> Cmp.max([1, nil, 2])\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nRaises an `Enum.EmptyError` on empty enumerables:\n\n    iex> Cmp.max([])\n    ** (Enum.EmptyError) empty error","ref":"Cmp.html#max/1-examples"},{"type":"function","title":"Cmp.max/2","doc":"Safe equivalent to `max/2`, which only works if both types are compatible and\nuses semantic comparison.","ref":"Cmp.html#max/2"},{"type":"function","title":"Examples - Cmp.max/2","doc":"iex> Cmp.max(1, 2)\n    2\n\n    iex> Cmp.max(1, 1.0)\n    1\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.max(1, nil)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nUnlike `max/2`, it will perform a semantic comparison for structs and not a\n[structural comparison](https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n    iex> Cmp.max(~D[2020-03-02], ~D[2019-06-06])\n    ~D[2020-03-02]\n\n    max(~D[2020-03-02], ~D[2019-06-06])\n    ~D[2019-06-06]","ref":"Cmp.html#max/2-examples"},{"type":"function","title":"Cmp.max_by/2","doc":"Safe equivalent of `Enum.max_by/3`, returning the element of a non-empty\nenumerable for which `fun` gives the maximum comparable value.","ref":"Cmp.html#max_by/2"},{"type":"function","title":"Examples - Cmp.max_by/2","doc":"iex> Cmp.max_by([%{x: 1}, %{x: 3}, %{x: 2}], & &1.x)\n    %{x: 3}\n\nRespects semantic comparison:\n\n    iex> Cmp.max_by([%{date: ~D[2020-03-02]}, %{date: ~D[2019-06-06]}], & &1.date)\n    %{date: ~D[2020-03-02]}\n\nRaises a `Cmp.TypeError` on non-uniform enumerables:\n\n    iex> Cmp.max_by([%{x: 1}, %{x: nil}, %{x: 2}], & &1.x)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nRaises an `Enum.EmptyError` on empty enumerables:\n\n    iex> Cmp.max_by([], & &1.x)\n    ** (Enum.EmptyError) empty error","ref":"Cmp.html#max_by/2-examples"},{"type":"function","title":"Cmp.min/1","doc":"Safe equivalent of `Enum.min/2`, returning the maximum of a non-empty\nenumerable of comparables.","ref":"Cmp.html#min/1"},{"type":"function","title":"Examples - Cmp.min/1","doc":"iex> Cmp.min([1, 3, 2])\n    1\n\nRespects semantic comparison:\n\n    iex> Cmp.min([~D[2020-03-02], ~D[2019-06-06]])\n    ~D[2019-06-06]\n\nRaises a `Cmp.TypeError` on non-uniform enumerables:\n\n    iex> Cmp.min([1, nil, 2])\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nRaises an `Enum.EmptyError` on empty enumerables:\n\n    iex> Cmp.min([])\n    ** (Enum.EmptyError) empty error","ref":"Cmp.html#min/1-examples"},{"type":"function","title":"Cmp.min/2","doc":"Safe equivalent to `min/2`, which only works if both types are compatible and\nuses semantic comparison.","ref":"Cmp.html#min/2"},{"type":"function","title":"Examples - Cmp.min/2","doc":"iex> Cmp.min(2, 1)\n    1\n\n    iex> Cmp.min(1, 1.0)\n    1\n\nIt will raise a `Cmp.TypeError` if trying to compare incompatible types\n\n    iex> Cmp.min(1, nil)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nUnlike `min/2`, it will perform a semantic comparison for structs and not a\n[structural comparison](https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n    iex> Cmp.min(~D[2020-03-02], ~D[2019-06-06])\n    ~D[2019-06-06]\n\n    min(~D[2020-03-02], ~D[2019-06-06])\n    ~D[2020-03-02]","ref":"Cmp.html#min/2-examples"},{"type":"function","title":"Cmp.min_by/2","doc":"Safe equivalent of `Enum.min_by/3`, returning the element of a non-empty\nenumerable for which `fun` gives the minimum comparable value.","ref":"Cmp.html#min_by/2"},{"type":"function","title":"Examples - Cmp.min_by/2","doc":"iex> Cmp.min_by([%{x: 1}, %{x: 3}, %{x: 2}], & &1.x)\n    %{x: 1}\n\nRespects semantic comparison:\n\n    iex> Cmp.min_by([%{date: ~D[2020-03-02]}, %{date: ~D[2019-06-06]}], & &1.date)\n    %{date: ~D[2019-06-06]}\n\nRaises a `Cmp.TypeError` on non-uniform enumerables:\n\n    iex> Cmp.min_by([%{x: 1}, %{x: nil}, %{x: 2}], & &1.x)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil\n\nRaises an `Enum.EmptyError` on empty enumerables:\n\n    iex> Cmp.min_by([], & &1.x)\n    ** (Enum.EmptyError) empty error","ref":"Cmp.html#min_by/2-examples"},{"type":"function","title":"Cmp.sort/2","doc":"Safe equivalent of `Enum.sort/2`.","ref":"Cmp.html#sort/2"},{"type":"function","title":"Examples - Cmp.sort/2","doc":"iex> Cmp.sort([3, 1, 2])\n    [1, 2, 3]\n\n    iex> Cmp.sort([3, 1, 2], :desc)\n    [3, 2, 1]\n\nRespects semantic comparison:\n\n    iex> Cmp.sort([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]])\n    [~D[2019-01-01], ~D[2019-06-06], ~D[2020-03-02]]\n\nRaises a `Cmp.TypeError` on non-uniform enumerables:\n\n    iex> Cmp.sort([~D[2019-01-01], nil, ~D[2020-03-02]])\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: ~D[2019-01-01], right: nil","ref":"Cmp.html#sort/2-examples"},{"type":"function","title":"Cmp.sort_by/3","doc":"Safe equivalent of `Enum.sort_by/2`.","ref":"Cmp.html#sort_by/3"},{"type":"function","title":"Examples - Cmp.sort_by/3","doc":"iex> Cmp.sort_by([%{x: 3}, %{x: 1}, %{x: 2}], & &1.x)\n    [%{x: 1}, %{x: 2}, %{x: 3}]\n\n    iex> Cmp.sort_by([%{x: 3}, %{x: 1}, %{x: 2}], & &1.x, :desc)\n    [%{x: 3}, %{x: 2}, %{x: 1}]\n\nRespects semantic comparison:\n\n    iex> Cmp.sort_by([%{date: ~D[2020-03-02]}, %{date: ~D[2019-06-06]}], & &1.date)\n    [%{date: ~D[2019-06-06]}, %{date: ~D[2020-03-02]}]\n\nRaises a `Cmp.TypeError` on non-uniform enumerables:\n\n    iex> Cmp.sort_by([%{x: 3}, %{x: \"1\"}, %{x: 2}], & &1.x)\n    ** (Cmp.TypeError) Failed to compare incompatible types - left: 3, right: \"1\"","ref":"Cmp.html#sort_by/3-examples"},{"type":"protocol","title":"Cmp.Comparable","doc":"A protocol to define how elements of a type can be compared.\n\nThe simplest way to define it for a custom struct is to use @derive\nbased on semantical comparison of fields, in the given order:\n\n    defmodule MyStruct do\n      @derive {Cmp.Comparable, using: [:date, :id]}\n\n      defstruct [:id, :date]\n    end\n\n    Cmp.sort([\n      %MyStruct{date: ~D[2020-03-02], id: 100},\n      %MyStruct{date: ~D[2020-03-02], id: 101},\n      %MyStruct{date: ~D[2019-06-06], id: 300}\n    ])\n    [\n      %MyStruct{date: ~D[2019-06-06], id: 300},\n      %MyStruct{date: ~D[2020-03-02], id: 100},\n      %MyStruct{date: ~D[2020-03-02], id: 101}\n    ]\n\nFor existing structs that are already implementing a `compare/2`\nfunction returning `:eq | :lt | :gt`, the protocol can simply be\ndone by passing `using: :compare`.\n\n    require Protocol\n    Protocol.derive(Cmp.Comparable, MyStruct, using: :compare)\n\nThis is already done for the following modules:\n- `Date`\n- `Time`\n- `DateTime`\n- `NaiveDateTime`\n- `Version`\n- `Decimal` (if available)","ref":"Cmp.Comparable.html"},{"type":"function","title":"Cmp.Comparable.compare/2","doc":"Defines how to semantically compare two elements of a given type.\n\nThis should return:\n- `:eq` if `left == right`\n- `:lt` if `left   right`","ref":"Cmp.Comparable.html#compare/2"},{"type":"type","title":"Cmp.Comparable.t/0","doc":"All the types that implement this protocol.","ref":"Cmp.Comparable.html#t:t/0"},{"type":"module","title":"Cmp.Util","doc":"","ref":"Cmp.Util.html"},{"type":"function","title":"Cmp.Util.comparable_structs/0","doc":"","ref":"Cmp.Util.html#comparable_structs/0"},{"type":"function","title":"Cmp.Util.compare_terms/2","doc":"","ref":"Cmp.Util.html#compare_terms/2"},{"type":"exception","title":"Cmp.TypeError","doc":"Error raised when trying to compare values of different types.","ref":"Cmp.TypeError.html"},{"type":"extras","title":"Cmp","doc":"# Cmp\n\n[![Hex Version](https://img.shields.io/hexpm/v/cmp.svg)](https://hex.pm/packages/cmp)\n[![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/cmp/)\n[![CI](https://github.com/sabiwara/cmp/workflows/CI/badge.svg)](https://github.com/sabiwara/cmp/actions?query=workflow%3ACI)\n\nSemantic comparison and sorting for Elixir.","ref":"readme.html"},{"type":"extras","title":"Why `Cmp`? - Cmp","doc":"The built-in comparison operators as well as functions like `Enum.sort/2` or\n`Enum.max/1` are based on Erlang's term ordering and suffer two issues, which\nrequire attention and might lead to unexpected behaviors or bugs:","ref":"readme.html#why-cmp"},{"type":"extras","title":"1. Structural comparisons - Cmp","doc":"Built-ins use\n[structural comparison over semantic comparison](https://hexdocs.pm/elixir/Kernel.html#module-structural-comparison):\n\n```elixir\niex> ~D[2020-03-02] > ~D[2019-06-06]\nfalse\n\niex> Enum.sort([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]])\n[~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]]\n```\n\nSemantic comparison is available but not straightforward:\n\n```elixir\niex> Date.compare(~D[2019-01-01], ~D[2020-03-02])\n:lt\n\niex> Enum.sort([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]], Date)\n[~D[2019-01-01], ~D[2019-06-06], ~D[2020-03-02]]\n```\n\n`Cmp` does the right thing out of the box:\n\n```elixir\niex> Cmp.gt?(~D[2020-03-02], ~D[2019-06-06])\ntrue\n\niex> Cmp.sort([~D[2019-01-01], ~D[2020-03-02], ~D[2019-06-06]])\n[~D[2019-01-01], ~D[2019-06-06], ~D[2020-03-02]]\n```","ref":"readme.html#1-structural-comparisons"},{"type":"extras","title":"2. Weakly typed - Cmp","doc":"Built-in comparators accept any set of operands:\n\n```elixir\niex> 2 < \"1\"\ntrue\n\niex> 0   false   Cmp.lte?(1, 1.0)\ntrue\n\niex> Cmp.lte?(2, \"1\")\n** (Cmp.TypeError) Failed to compare incompatible types - left: 2, right: \"1\"\n```","ref":"readme.html#2-weakly-typed"},{"type":"extras","title":"Installation - Cmp","doc":"`Cmp` can be installed by adding `cmp` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cmp, \"~> 0.1.3\"}\n  ]\nend\n```\n\nThe documentation can be found at\n[https://hexdocs.pm/cmp](https://hexdocs.pm/cmp).","ref":"readme.html#installation"},{"type":"extras","title":"Design goals - Cmp","doc":"- Fast and well-optimized - the overhead should be quite small over built-in\n  equivalents. See the `benchmarks/` folder for more details.\n- No need to require macros, plain functions\n- Easily extensible through the `Cmp.Comparable` protocol\n- Robust and well-tested (both unit and property-based)\n\nSupporting comparisons between non-homogeneous types such as mixed `Decimal` and\nbuilt-in numbers for instance is a non-goal. This limitation is a necessary\ntrade-off in order to ensure the points above. Use the `Decimal` library\ndirectly if you need this.","ref":"readme.html#design-goals"},{"type":"extras","title":"Copyright and License - Cmp","doc":"Cmp is licensed under the [MIT License](LICENSE.md).","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Dev - Changelog","doc":"","ref":"changelog.html#dev"},{"type":"extras","title":"v0.1.3 (2024-05-04) - Changelog","doc":"- Update `:ex_doc`","ref":"changelog.html#v0-1-3-2024-05-04"},{"type":"extras","title":"v0.1.2 (2023-03-17) - Changelog","doc":"- Add `Cmp.sort_by/2`\n- Add `Cmp.min_by/2` and `Cmp.max_by/2`","ref":"changelog.html#v0-1-2-2023-03-17"},{"type":"extras","title":"v0.1.1 (2023-03-05) - Changelog","doc":"- Fix broken link in doc","ref":"changelog.html#v0-1-1-2023-03-05"},{"type":"extras","title":"v0.1.0 (2023-03-04) - Changelog","doc":"- Initial release","ref":"changelog.html#v0-1-0-2023-03-04"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2023 Sabiwara\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of\nthis software and associated documentation files (the \"Software\"), to deal in\nthe Software without restriction, including without limitation the rights to\nuse, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of\nthe Software, and to permit persons to whom the Software is furnished to do so,\nsubject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS\nFOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR\nCOPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER\nIN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN\nCONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}